@@ -65,7 +65,7 @@ elif _sys.version_info >= (3, 8, 0) and _platform.system() == 'Darwin':
 # this is managed by git tag and updated on every release
 # NOTE: this represents the NEXT release version
 
-__version__ = '3.9.3'
+__version__ = '3.11.1'
 
 # do not change this line manually
 # this is managed by proto/build-proto.sh and updated on every execution
@@ -102,6 +102,11 @@ __jina_env__ = (
     'JINA_OPTOUT_TELEMETRY',
     'JINA_RANDOM_PORT_MAX',
     'JINA_RANDOM_PORT_MIN',
+    'JINA_LOCKS_ROOT',
+    'JINA_OPTOUT_TELEMETRY',
+    'JINA_K8S_ACCESS_MODES',
+    'JINA_K8S_STORAGE_CLASS_NAME',
+    'JINA_K8S_STORAGE_CAPACITY',
 )
 
 __default_host__ = _os.environ.get(
@@ -109,6 +114,10 @@ __default_host__ = _os.environ.get(
 )
 __docker_host__ = 'host.docker.internal'
 __default_executor__ = 'BaseExecutor'
+__default_gateway__ = 'BaseGateway'
+__default_http_gateway__ = 'HTTPGateway'
+__default_websocket_gateway__ = 'WebSocketGateway'
+__default_grpc_gateway__ = 'GRPCGateway'
 __default_endpoint__ = '/default'
 __ready_msg__ = 'ready and listening'
 __stop_msg__ = 'terminated'
@@ -204,5 +213,8 @@ from jina.orchestrate.flow.base import Flow
 from jina.serve.executors import BaseExecutor as Executor
 from jina.serve.executors.decorators import monitor, requests
 
+# Custom Gateway
+from jina.serve.gateway import BaseGateway as Gateway
+
 __all__ = [_s for _s in dir() if not _s.startswith('_')]
 __all__.extend(_names_with_underscore)
