@@ -1,4 +1,5 @@
 import asyncio
+import base64
 import functools
 import inspect
 import json
@@ -9,6 +10,7 @@ import re
 import sys
 import threading
 import time
+import urllib
 import uuid
 import warnings
 from argparse import ArgumentParser, Namespace
@@ -65,6 +67,7 @@ __all__ = [
 ]
 
 T = TypeVar('T')
+GATEWAY_NAME = 'gateway'
 
 
 def deprecated_alias(**aliases):
@@ -284,9 +287,9 @@ def parse_arg(v: str) -> Optional[Union[bool, int, str, list, float]]:
 
     if v.startswith('[') and v.endswith(']'):
         # function args must be immutable tuples not list
-        tmp = v.replace('[', '').replace(']', '').strip().split(',')
+        tmp = v.replace('[', '').replace(']', '').strip()
         if len(tmp) > 0:
-            return [parse_arg(vv.strip()) for vv in tmp]
+            return [parse_arg(vv.strip()) for vv in tmp.split(',')]
         else:
             return []
     try:
@@ -798,6 +801,7 @@ class ArgNamespace:
         """
         args = []
         from jina.serve.executors import BaseExecutor
+        from jina.serve.gateway import BaseGateway
 
         for k, v in kwargs.items():
             k = k.replace('_', '-')
@@ -811,6 +815,8 @@ class ArgNamespace:
                     args.extend([f'--{k}', json.dumps(v)])
                 elif isinstance(v, type) and issubclass(v, BaseExecutor):
                     args.extend([f'--{k}', v.__name__])
+                elif isinstance(v, type) and issubclass(v, BaseGateway):
+                    args.extend([f'--{k}', v.__name__])
                 else:
                     args.extend([f'--{k}', str(v)])
         return args
@@ -930,6 +936,15 @@ def get_full_version() -> Optional[Tuple[Dict, Dict]]:
     from google.protobuf.internal import api_implementation
     from grpc import _grpcio_metadata
 
+    try:
+        from hubble import __version__ as __hubble_version__
+    except:
+        __hubble_version__ = 'not-available'
+    try:
+        from jcloud import __version__ as __jcloud_version__
+    except:
+        __jcloud_version__ = 'not-available'
+
     from jina import (
         __docarray_version__,
         __jina_env__,
@@ -945,9 +960,11 @@ def get_full_version() -> Optional[Tuple[Dict, Dict]]:
         info = {
             'jina': __version__,
             'docarray': __docarray_version__,
+            'jcloud': __jcloud_version__,
+            'jina-hubble-sdk': __hubble_version__,
             'jina-proto': __proto_version__,
             'protobuf': google.protobuf.__version__,
-            'proto-backend': api_implementation._default_implementation_type,
+            'proto-backend': api_implementation.Type(),
             'grpcio': getattr(grpc, '__version__', _grpcio_metadata.__version__),
             'pyyaml': yaml.__version__,
             'python': platform.python_version(),
@@ -960,6 +977,8 @@ def get_full_version() -> Optional[Tuple[Dict, Dict]]:
             'session-id': str(random_uuid(use_uuid1=True)),
             'uptime': __uptime__,
             'ci-vendor': get_ci_vendor() or __unset_msg__,
+            'internal': 'jina-ai'
+            in os.getenv('GITHUB_ACTION_REPOSITORY', __unset_msg__),
         }
 
         env_info = {k: os.getenv(k, __unset_msg__) for k in __jina_env__}
@@ -1445,7 +1464,7 @@ def dunder_get(_dict: Any, key: str) -> Any:
     return dunder_get(result, part2) if part2 else result
 
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from fastapi import FastAPI
 
 
@@ -1561,7 +1580,7 @@ def _parse_kwargs(kwargs: Dict[str, Any]) -> Dict[str, Any]:
             return_scheme['port'],
             return_scheme['protocol'],
             return_scheme['tls'],
-        ) = _parse_host_scheme(kwargs['host'])
+        ) = parse_host_scheme(kwargs['host'])
 
         for key, value in return_scheme.items():
             if value:
@@ -1584,7 +1603,7 @@ def _delete_host_slash(kwargs: Dict[str, Any]) -> Dict[str, Any]:
     return kwargs
 
 
-def _parse_host_scheme(host: str) -> Tuple[str, str, str, bool]:
+def parse_host_scheme(host: str) -> Tuple[str, str, str, bool]:
     scheme, _hostname, port = _parse_url(host)
 
     tls = None
@@ -1612,7 +1631,7 @@ def _parse_url(host):
     return scheme, host, port
 
 
-def is_port_free(host, port):
+def _single_port_free(host: str, port: int) -> bool:
     with socket(AF_INET, SOCK_STREAM) as session:
         if session.connect_ex((host, port)) == 0:
             return False
@@ -1620,6 +1639,13 @@ def is_port_free(host, port):
             return True
 
 
+def is_port_free(host: str, port: Union[int, List[int]]) -> bool:
+    if isinstance(port, list):
+        return all([_single_port_free(host, _p) for _p in port])
+    else:
+        return _single_port_free(host, port)
+
+
 def _parse_ports(port: str) -> Union[int, List]:
     """Parse port
 
@@ -1631,7 +1657,7 @@ def _parse_ports(port: str) -> Union[int, List]:
         _parse_port('8000')
         8000
 
-        _parse_port('8001:8002:8005')
+        _parse_port('8001,8002,8005')
         [80001, 8002, 8005]
 
     :param port: the string to parse
@@ -1642,6 +1668,93 @@ def _parse_ports(port: str) -> Union[int, List]:
     except ValueError as e:
         if ',' in port:
             port = [int(port_) for port_ in port.split(',')]
-        else:
+        elif not isinstance(port, list):
             raise e
     return port
+
+
+def _parse_hosts(host: str) -> Union[str, List[str]]:
+    """Parse port
+
+    EXAMPLE USAGE
+
+    .. code-block:: python
+
+
+        _parse_hosts('localhost')
+        'localhost'
+
+        _parse_port('localhost,91.198.174.192')
+        ['localhost', '91.198.174.192']
+
+    :param host: the string to parse
+    :return: the host or the iterable of hosts
+    """
+    hosts = host.split(',')
+    return hosts[0] if len(hosts) == 1 else hosts
+
+
+def send_telemetry_event(event: str, obj: Any, **kwargs) -> None:
+    """Sends in a thread a request with telemetry for a given event
+    :param event: Event leading to the telemetry entry
+    :param obj: Object to be tracked
+    :param kwargs: Extra kwargs to be passed to the data sent
+    """
+
+    if 'JINA_OPTOUT_TELEMETRY' in os.environ:
+        return
+
+    def _telemetry():
+        url = 'https://telemetry.jina.ai/'
+        try:
+            from jina.helper import get_full_version
+
+            metas, _ = get_full_version()
+            data = base64.urlsafe_b64encode(
+                json.dumps(
+                    {**metas, 'event': f'{obj.__class__.__name__}.{event}', **kwargs}
+                ).encode('utf-8')
+            )
+
+            req = urllib.request.Request(
+                url, data=data, headers={'User-Agent': 'Mozilla/5.0'}
+            )
+            urllib.request.urlopen(req)
+
+        except:
+            pass
+
+    threading.Thread(target=_telemetry, daemon=True).start()
+
+
+def make_iterable(o: object) -> Iterable:
+    """
+    Make an object an iterable by wrapping it as a singleton list.
+    If the input is already an iterable (except str and bytes), it will be returned as is.
+    Str and bytes are treated as non-iterable, and thus wrapped in a list.
+
+    EXAMPLE USAGE
+
+    .. code-block:: python
+
+
+        make_iter(1)
+        [1]
+
+        make_iter('a')
+        ['a']
+
+        make_iter([1, 2, 3])
+        [1, 2, 3]
+
+        make_iter((1, 2, 3))
+        (1, 2, 3)
+
+
+    :param o: the object to be converted to an iterable
+    :return: the iterable
+    """
+    if isinstance(o, Iterable) and not isinstance(o, (str, bytes)):
+        return o
+    else:
+        return [o]
