@@ -1,12 +1,13 @@
 import json
+import os
 from argparse import Namespace
-from typing import Dict, Optional, Tuple, Union, List
+from typing import Dict, List, Optional, Tuple, Union
 
-from jina.serve.networking import K8sGrpcConnectionPool
 from jina.orchestrate.deployments.config.k8slib import kubernetes_tools
+from jina.serve.networking import GrpcConnectionPool
 
 
-def get_deployment_yamls(
+def get_template_yamls(
     name: str,
     namespace: str,
     image_name: str,
@@ -17,7 +18,7 @@ def get_deployment_yamls(
     jina_deployment_name: str,
     pod_type: str,
     shard_id: Optional[int] = None,
-    port: Optional[int] = None,
+    port: Optional[Union[int, List[int]]] = None,
     env: Optional[Dict] = None,
     gpus: Optional[Union[int, str]] = None,
     image_name_uses_before: Optional[str] = None,
@@ -26,6 +27,10 @@ def get_deployment_yamls(
     container_cmd_uses_after: Optional[str] = None,
     container_args_uses_before: Optional[str] = None,
     container_args_uses_after: Optional[str] = None,
+    monitoring: bool = False,
+    port_monitoring: Optional[int] = None,
+    protocol: Optional[Union[str, List[str]]] = None,
+    volumes: Optional[List[str]] = None,
 ) -> List[Dict]:
     """Get the yaml description of a service on Kubernetes
 
@@ -48,23 +53,40 @@ def get_deployment_yamls(
     :param container_cmd_uses_after: command executed in the uses_after container on the k8s pods
     :param container_args_uses_before: arguments used for uses_before container on the k8s pod
     :param container_args_uses_after: arguments used for uses_after container on the k8s pod
+    :param monitoring: enable monitoring on the deployment
+    :param port_monitoring: port which will be exposed, for the prometheus server, by the deployed containers
+    :param protocol: In case of being a Gateway, the protocol or protocols list used to expose its server
+    :param volumes: If volumes are passed to Executors, Jina will create a StatefulSet instead of Deployment and include the first volume in the volume mounts
     :return: Return a dictionary with all the yaml configuration needed for a deployment
     """
     # we can always assume the ports are the same for all executors since they run on different k8s pods
     # port expose can be defined by the user
     if not port:
-        port = K8sGrpcConnectionPool.K8S_PORT
+        port = GrpcConnectionPool.K8S_PORT
 
-    deployment_params = {
+    if not port_monitoring:
+        port_monitoring = GrpcConnectionPool.K8S_PORT_MONITORING
+
+    # we cast port to list of ports and protocol to list of protocols
+    if not isinstance(port, list):
+        ports = [port]
+    else:
+        ports = port
+    if not isinstance(protocol, list):
+        protocols = [protocol]
+    else:
+        protocols = protocol
+
+    template_params = {
         'name': name,
         'namespace': namespace,
         'image': image_name,
         'replicas': replicas,
         'command': container_cmd,
         'args': container_args,
-        'port': port,
-        'port_uses_before': K8sGrpcConnectionPool.K8S_PORT_USES_BEFORE,
-        'port_uses_after': K8sGrpcConnectionPool.K8S_PORT_USES_AFTER,
+        'port': ports[0],
+        'port_uses_before': GrpcConnectionPool.K8S_PORT_USES_BEFORE,
+        'port_uses_after': GrpcConnectionPool.K8S_PORT_USES_AFTER,
         'args_uses_before': container_args_uses_before,
         'args_uses_after': container_args_uses_after,
         'command_uses_before': container_cmd_uses_before,
@@ -75,33 +97,88 @@ def get_deployment_yamls(
         'jina_deployment_name': jina_deployment_name,
         'shard_id': f'\"{shard_id}\"' if shard_id is not None else '\"\"',
         'pod_type': pod_type,
+        'protocol': str(protocols[0]).lower() if protocols[0] is not None else '',
+        'volume_path': volumes[0] if volumes is not None else None,
     }
 
     if gpus:
-        deployment_params['device_plugins'] = {'nvidia.com/gpu': gpus}
+        template_params['device_plugins'] = {'nvidia.com/gpu': gpus}
 
-    template_name = 'deployment'
+    template_name = 'deployment-executor' if name != 'gateway' else 'deployment-gateway'
 
-    if image_name_uses_before and image_name_uses_after:
+    template_params['ports-section'] = ''.join(
+        [f'\n            - containerPort: {_p}' for _p in ports]
+    )
+
+    if volumes:
+        template_name = 'statefulset-executor'
+        template_params['accessModes'] = json.loads(
+            os.environ.get('JINA_K8S_ACCESS_MODES', '["ReadWriteOnce"]')
+        )
+        template_params['storageClassName'] = os.environ.get(
+            'JINA_K8S_STORAGE_CLASS_NAME', 'standard'
+        )
+        template_params['storageCapacity'] = os.environ.get(
+            'JINA_K8S_STORAGE_CAPACITY', '10G'
+        )
+    elif image_name_uses_before and image_name_uses_after:
         template_name = 'deployment-uses-before-after'
     elif image_name_uses_before:
         template_name = 'deployment-uses-before'
     elif image_name_uses_after:
         template_name = 'deployment-uses-after'
 
+    if monitoring:
+        service_yaml = kubernetes_tools.get_yaml(
+            'service_monitoring',
+            {
+                'name': name,
+                'target': name,
+                'namespace': namespace,
+                'port': ports[0],
+                'type': 'ClusterIP',
+                'port_monitoring': port_monitoring,
+            },
+        )
+
+        service_monitor_yaml = kubernetes_tools.get_yaml(
+            'service_monitor',
+            {
+                'name': name,
+                'target': name,
+                'namespace': namespace,
+            },
+        )
+    else:
+        service_yaml = kubernetes_tools.get_yaml(
+            'service',
+            {
+                'name': name,
+                'target': name,
+                'namespace': namespace,
+                'port': ports[0],
+                'type': 'ClusterIP',
+            },
+        )
+        service_monitor_yaml = None
+
+    extra_services = [
+        kubernetes_tools.get_yaml(
+            'service',
+            {
+                'name': f'{name}-{i}-{protocol}'.lower(),
+                'target': name,
+                'namespace': namespace,
+                'port': port,
+                'type': 'ClusterIP',
+            },
+        )
+        for i, (port, protocol) in enumerate(zip(ports[1:], protocols[1:]), start=1)
+    ]
+
+    template_yaml = kubernetes_tools.get_yaml(template_name, template_params)
+
     yamls = [
-        kubernetes_tools.get_yaml(
-            'connection-pool-role',
-            {
-                'namespace': namespace,
-            },
-        ),
-        kubernetes_tools.get_yaml(
-            'connection-pool-role-binding',
-            {
-                'namespace': namespace,
-            },
-        ),
         kubernetes_tools.get_yaml(
             'configmap',
             {
@@ -110,19 +187,14 @@ def get_deployment_yamls(
                 'data': env,
             },
         ),
-        kubernetes_tools.get_yaml(
-            'service',
-            {
-                'name': name,
-                'target': name,
-                'namespace': namespace,
-                'port': port,
-                'type': 'ClusterIP',
-            },
-        ),
-        kubernetes_tools.get_yaml(template_name, deployment_params),
+        service_yaml,
+        *extra_services,
+        template_yaml,
     ]
 
+    if service_monitor_yaml:
+        yamls.append(service_monitor_yaml)
+
     return yamls
 
 
@@ -158,8 +230,6 @@ def get_cli_params(
     cli_args = []
     for attribute, cli_attribute, value in arg_list:
         # TODO: This should not be here, its a workaround for our parser design with boolean values
-        if attribute == 'k8s_connection_pool' and not value:
-            cli_args.append(f'"--k8s-disable-connection-pool"')
         if attribute in skip_attributes:
             continue
         if type(value) == bool and value:
