@@ -17,6 +17,8 @@ from jina.serve.runtimes.asyncio import AsyncNewLoopRuntime
 
 __all__ = ['BasePod', 'Pod']
 
+from jina.serve.runtimes.gateway import GatewayRuntime
+
 
 def run(
     args: 'argparse.Namespace',
@@ -91,7 +93,7 @@ def run(
     finally:
         _unset_envs()
         is_shutdown.set()
-        logger.debug(f' Process terminated')
+        logger.debug(f'process terminated')
 
 
 class BasePod(ABC):
@@ -109,7 +111,7 @@ class BasePod(ABC):
 
         if hasattr(self.args, 'port'):
             self.args.port = self.args.port
-        self.args.parallel = self.args.shards
+        self.args.parallel = getattr(self.args, 'shards', 1)
         self.name = self.args.name or self.__class__.__name__
         self.is_forked = False
         self.logger = JinaLogger(self.name, **vars(self.args))
@@ -134,6 +136,8 @@ class BasePod(ABC):
         self._timeout_ctrl = self.args.timeout_ctrl
 
     def _get_control_address(self):
+        if self.args.pod_role == PodRoleType.GATEWAY:
+            return f'{self.args.host}:{self.args.port[0]}'
         return f'{self.args.host}:{self.args.port}'
 
     def close(self) -> None:
@@ -170,7 +174,7 @@ class BasePod(ABC):
             self.logger.debug(
                 f'{"shutdown is is already set" if self.is_shutdown.is_set() else "Runtime was never started"}. Runtime will end gracefully on its own'
             )
-            pass
+            self._terminate()
         self.is_shutdown.set()
         self.logger.debug(__stop_msg__)
         self.logger.close()
@@ -188,12 +192,21 @@ class BasePod(ABC):
         :param timeout: The time to wait before readiness or failure is determined
             .. # noqa: DAR201
         """
-        return AsyncNewLoopRuntime.wait_for_ready_or_shutdown(
-            timeout=timeout,
-            ready_or_shutdown_event=self.ready_or_shutdown.event,
-            ctrl_address=self.runtime_ctrl_address,
-            timeout_ctrl=self._timeout_ctrl,
-        )
+        if self.args.pod_role == PodRoleType.GATEWAY:
+            return GatewayRuntime.wait_for_ready_or_shutdown(
+                timeout=timeout,
+                ready_or_shutdown_event=self.ready_or_shutdown.event,
+                ctrl_address=self.runtime_ctrl_address,
+                timeout_ctrl=self._timeout_ctrl,
+                protocol=self.args.protocol[0],
+            )
+        else:
+            return AsyncNewLoopRuntime.wait_for_ready_or_shutdown(
+                timeout=timeout,
+                ready_or_shutdown_event=self.ready_or_shutdown.event,
+                ctrl_address=self.runtime_ctrl_address,
+                timeout_ctrl=self._timeout_ctrl,
+            )
 
     def _fail_start_timeout(self, timeout):
         """
@@ -338,9 +351,9 @@ class Pod(BasePod):
         :param args: extra positional arguments to pass to join
         :param kwargs: extra keyword arguments to pass to join
         """
-        self.logger.debug(f' Joining the process')
+        self.logger.debug(f'joining the process')
         self.worker.join(*args, **kwargs)
-        self.logger.debug(f' Successfully joined the process')
+        self.logger.debug(f'successfully joined the process')
 
     def _terminate(self):
         """Terminate the Pod.
@@ -348,7 +361,7 @@ class Pod(BasePod):
         """
         self.logger.debug(f'terminating the runtime process')
         self.worker.terminate()
-        self.logger.debug(f' runtime process properly terminated')
+        self.logger.debug(f'runtime process properly terminated')
 
     def _get_runtime_cls(self) -> AsyncNewLoopRuntime:
         from jina.orchestrate.pods.helper import update_runtime_cls
