@@ -2,15 +2,14 @@ from argparse import Namespace
 from copy import deepcopy
 from typing import TYPE_CHECKING, Type
 
-from jina import __default_host__
+from hubble.executor.helper import is_valid_huburi
+from hubble.executor.hubio import HubIO
+
 from jina.enums import PodRoleType
-from jina.hubble.helper import is_valid_huburi
-from jina.hubble.hubio import HubIO
 from jina.orchestrate.pods import Pod
 from jina.orchestrate.pods.container import ContainerPod
-from jina.orchestrate.pods.jinad import JinaDPod
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from jina.orchestrate.pods import BasePod
 
 
@@ -29,9 +28,6 @@ class PodFactory:
         """
         # copy to update but forward original
         cargs = deepcopy(args)
-        if cargs.host != __default_host__:
-            cargs.timeout_ready = -1
-            return JinaDPod(cargs)
 
         if is_valid_huburi(cargs.uses):
             _hub_args = deepcopy(args)
@@ -40,9 +36,9 @@ class PodFactory:
             cargs.uses = HubIO(_hub_args).pull()
 
         if (
-            cargs.pod_role != PodRoleType.HEAD
-            and cargs.uses
-            and cargs.uses.startswith('docker://')
+                cargs.pod_role != PodRoleType.HEAD
+                and cargs.uses
+                and cargs.uses.startswith('docker://')
         ):
             return ContainerPod(cargs)
         else:
