@@ -1,18 +1,15 @@
 import multiprocessing
-import threading
 from copy import deepcopy
 from functools import partial
-from typing import TYPE_CHECKING, Callable, Dict, Optional, Union
+from typing import TYPE_CHECKING
 
-from grpc import RpcError
+from hubble.executor.helper import is_valid_huburi
+from hubble.executor.hubio import HubIO
 
-from jina.enums import GatewayProtocolType, PodRoleType
-from jina.hubble.helper import is_valid_huburi
-from jina.hubble.hubio import HubIO
-from jina.serve.networking import GrpcConnectionPool
-from jina.types.request.control import ControlRequest
+from jina.enums import PodRoleType
+from jina.parsers.helper import _set_gateway_uses
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from argparse import Namespace
 
 
@@ -78,35 +75,16 @@ def update_runtime_cls(args, copy=False) -> 'Namespace':
     :return: runtime class as a string
     """
     _args = deepcopy(args) if copy else args
-    gateway_runtime_dict = {
-        GatewayProtocolType.GRPC: 'GRPCGatewayRuntime',
-        GatewayProtocolType.WEBSOCKET: 'WebSocketGatewayRuntime',
-        GatewayProtocolType.HTTP: 'HTTPGatewayRuntime',
-    }
+
     if _args.runtime_cls == 'WorkerRuntime' and is_valid_huburi(_args.uses):
         _hub_args = deepcopy(_args)
         _hub_args.uri = _args.uses
         _hub_args.no_usage = True
         _args.uses = HubIO(_hub_args).pull()
 
-    if hasattr(_args, 'protocol'):
-        _args.runtime_cls = gateway_runtime_dict[_args.protocol]
+    if hasattr(_args, 'protocol') and _args.pod_role == PodRoleType.GATEWAY:
+        _set_gateway_uses(_args)
     if _args.pod_role == PodRoleType.HEAD:
         _args.runtime_cls = 'HeadRuntime'
 
     return _args
-
-
-def is_ready(address: str) -> bool:
-    """
-    TODO: make this async
-    Check if status is ready.
-    :param address: the address where the control message needs to be sent
-    :return: True if status is ready else False.
-    """
-
-    try:
-        GrpcConnectionPool.send_request_sync(ControlRequest('STATUS'), address)
-    except RpcError:
-        return False
-    return True
