@@ -1,16 +1,13 @@
 """Module for helper functions in the parser"""
+
 import argparse
 import os
-import warnings
-from typing import Tuple, Type
+from typing import Tuple
+
+from jina.enums import GatewayProtocolType
+from jina.logging.predefined import default_logger
 
 _SHOW_ALL_ARGS = 'JINA_FULL_CLI' in os.environ
-if _SHOW_ALL_ARGS:
-    from jina.logging.predefined import default_logger
-
-    default_logger.warning(
-        f'Setting {_SHOW_ALL_ARGS} will make remote Pods with sharding not work when using JinaD'
-    )
 
 
 def add_arg_group(parser, title):
@@ -265,34 +262,50 @@ class _ColoredHelpFormatter(argparse.ArgumentDefaultsHelpFormatter):
         return lines
 
 
-_chf = _ColoredHelpFormatter
-
-
-class DeprecateAction(argparse.Action):
-    """
-    Action to deprecated an argument in argparse
-    """
-
-    def __call__(self, parser, namespace, values, option_string=None):  # noqa
-        warnings.warn('Argument {self.option_strings} is deprecated and is *ignored*')
-        delattr(namespace, self.dest)
-
-
-def get_deprecation_renamed_action(
-    replacement: str, action: Type[argparse.Action] = argparse.Action
-):
-    """
-    To deprecate an argument when it has been renamed for argparse
-
-    :param replacement: the new argument name
-    :param action: class of the action that you want to overload with the deprecation to keep the old behavior
-    :return: a action class that can
-    """
-
-    class _DeprecateRenamedAction(action):
-        def __call__(self, parser, namespace, values, option_string=None):
-            warnings.warn(
-                f'Argument {self.option_strings} is deprecated, please use {replacement} instead'
+def _set_gateway_uses(args: 'argparse.Namespace'):
+    gateway_dict = {
+        GatewayProtocolType.GRPC: 'GRPCGateway',
+        GatewayProtocolType.WEBSOCKET: 'WebSocketGateway',
+        GatewayProtocolType.HTTP: 'HTTPGateway',
+    }
+    if not args.uses:
+        if len(args.protocol) == 1:
+            args.uses = gateway_dict[args.protocol[0]]
+        else:
+            raise ValueError(
+                'You need to specify exactly 1 protocol if you want to use a jina built-in gateway'
             )
 
-    return _DeprecateRenamedAction
+
+class CastToIntAction(argparse.Action):
+    """argparse action to cast a list of values to int"""
+
+    def __call__(self, parser, args, values, option_string=None):
+        """
+        call the CastToIntAction
+
+
+        .. # noqa: DAR401
+        :param parser: the parser
+        :param args: args to initialize the values
+        :param values: the values to add to the parser
+        :param option_string: inherited, not used
+        """
+        if isinstance(values, list):
+            d = [_port_to_int(port) for port in values]
+        else:
+            d = _port_to_int(values)
+        setattr(args, self.dest, d)
+
+
+def _port_to_int(port):
+    try:
+        return int(port)
+    except ValueError:
+        default_logger.warning(
+            f'port {port} is not an integer and cannot be cast to one'
+        )
+        return port
+
+
+_chf = _ColoredHelpFormatter
