@@ -2,6 +2,7 @@
 
 from jina import __default_host__, helper
 from jina.parsers.helper import KVAppendAction, add_arg_group
+from jina.parsers.orchestrate.runtimes.runtime import mixin_base_runtime_parser
 
 
 def mixin_remote_runtime_parser(parser):
@@ -44,13 +45,50 @@ def mixin_gateway_parser(parser):
     _add_proxy(gp)
 
     gp.add_argument(
-        '--port-expose',
-        type=int,
-        dest='port',
-        default=helper.random_port(),
-        help='The port that the gateway exposes for clients for GRPC connections.',
+        '--uses',
+        type=str,
+        default=None,
+        # TODO: add Jina Hub Gateway
+        help='''
+        The config of the gateway, it could be one of the followings:
+        * the string literal of an Gateway class name
+        * a Gateway YAML file (.yml, .yaml, .jaml)
+        * a docker image (must start with `docker://`)
+        * the string literal of a YAML config (must start with `!` or `jtype: `)
+        * the string literal of a JSON config
+
+        When use it under Python, one can use the following values additionally:
+        - a Python dict that represents the config
+        - a text file stream has `.read()` interface
+        ''',
     )
 
+    gp.add_argument(
+        '--uses-with',
+        action=KVAppendAction,
+        metavar='KEY: VALUE',
+        nargs='*',
+        help='''
+    Dictionary of keyword arguments that will override the `with` configuration in `uses`
+    ''',
+    )
+
+    gp.add_argument(
+        '--py-modules',
+        type=str,
+        nargs='*',
+        metavar='PATH',
+        help='''
+The customized python modules need to be imported before loading the gateway
+
+Note that the recommended way is to only import a single module - a simple python file, if your
+gateway can be defined in a single file, or an ``__init__.py`` file if you have multiple files,
+which should be structured as a python package.
+''',
+    )
+
+    mixin_base_runtime_parser(gp)
+
     parser.add_argument(
         '--graph-description',
         type=str,
@@ -68,11 +106,19 @@ def mixin_gateway_parser(parser):
     parser.add_argument(
         '--deployments-addresses',
         type=str,
-        help='dictionary JSON with the input addresses of each Deployment',
+        help='JSON dictionary with the input addresses of each Deployment',
         default='{}',
     )
 
     parser.add_argument(
+        '--deployments-metadata',
+        type=str,
+        help='JSON dictionary with the request metadata for each Deployment',
+        default='{}',
+    )
+
+    parser.add_argument(
+        '--deployments-no-reduce',
         '--deployments-disable-reduce',
         type=str,
         help='list JSON disabling the built-in merging mechanism for each Deployment listed',
@@ -94,12 +140,34 @@ def mixin_gateway_parser(parser):
     )
 
 
+def mixin_gateway_protocol_parser(parser):
+    """Add the arguments for the protocol to the gateway parser
+
+    :param parser: the parser configure
+    """
+
+    from jina.enums import GatewayProtocolType
+
+    parser.add_argument(
+        '--protocol',
+        '--protocols',
+        nargs='+',
+        type=GatewayProtocolType.from_string,
+        choices=list(GatewayProtocolType),
+        default=[GatewayProtocolType.GRPC],
+        help=f'Possible communication protocols between server and client. Depending on your chosen gateway, choose the convenient protocols from: {[protocol.to_string() for protocol in list(GatewayProtocolType)]}.',
+    )
+
+
 def _add_host(arg_group):
     arg_group.add_argument(
         '--host',
+        '--host-in',
         type=str,
         default=__default_host__,
-        help=f'The host address of the runtime, by default it is {__default_host__}.',
+        help=f'The host address of the runtime, by default it is {__default_host__}.'
+        ' In the case of an external Executor (`--external` or `external=True`) this can be a list of hosts, separated by commas.'
+        ' Then, every resulting address will be considered as one replica of the Executor.',
     )
 
 
@@ -172,7 +240,7 @@ def mixin_http_gateway_parser(parser=None):
         help='''
         If set, `/index`, `/search`, `/update`, `/delete` endpoints are removed from HTTP interface.
 
-        Any executor that has `@requests(on=...)` bind with those values will receive data requests.
+        Any executor that has `@requests(on=...)` bound with those values will receive data requests.
         ''',
     )
 
@@ -197,16 +265,6 @@ More details can be found in Uvicorn docs: https://www.uvicorn.org/settings/
 ''',
     )
 
-    gp.add_argument(
-        '--grpc-server-kwargs',
-        action=KVAppendAction,
-        metavar='KEY: VALUE',
-        nargs='*',
-        help='''
-    Dictionary of kwargs arguments that will be passed to the grpc server when starting the server # todo update
-    ''',
-    )
-
     gp.add_argument(
         '--ssl-certfile',
         type=str,
@@ -235,9 +293,9 @@ def mixin_prefetch_parser(parser=None):
     gp.add_argument(
         '--prefetch',
         type=int,
-        default=0,
+        default=1000,
         help='''
     Number of requests fetched from the client before feeding into the first Executor. 
     
-    Used to control the speed of data input into a Flow. 0 disables prefetch (disabled by default)''',
+    Used to control the speed of data input into a Flow. 0 disables prefetch (1000 requests is the default)''',
     )
