@@ -1,4 +1,5 @@
 import contextlib
+import copy
 import inspect
 import multiprocessing
 import os
@@ -7,21 +8,22 @@ import warnings
 from types import SimpleNamespace
 from typing import TYPE_CHECKING, Any, Dict, Optional, Type, Union
 
-from jina import __args_executor_init__, __default_endpoint__
+from jina import __args_executor_init__, __cache_path__, __default_endpoint__
 from jina.enums import BetterEnum
 from jina.helper import ArgNamespace, T, iscoroutinefunction, typename
 from jina.importer import ImportExtensions
 from jina.jaml import JAML, JAMLCompatible, env_var_regex, internal_var_regex
 from jina.logging.logger import JinaLogger
 from jina.serve.executors.decorators import (
+    _init_requests_by_class,
     avoid_concurrent_lock_cls,
-    requests,
-    store_init_kwargs,
-    wrap_func,
 )
+from jina.serve.executors.metas import get_executor_taboo
+from jina.serve.helper import store_init_kwargs, wrap_func
+from jina.serve.instrumentation import MetricsTimer
 
-if TYPE_CHECKING:
-    from prometheus_client import Summary
+if TYPE_CHECKING:  # pragma: no cover
+    from opentelemetry.context.context import Context
 
 __dry_run_endpoint__ = '_jina_dry_run_'
 
@@ -39,6 +41,8 @@ class ExecutorType(type(JAMLCompatible), type):
         :return: Executor class
         """
         _cls = super().__new__(cls, *args, **kwargs)
+        # this needs to be here, in the case where Executors inherited do not define new `requests`
+        _init_requests_by_class(_cls)
         return cls.register_class(_cls)
 
     @staticmethod
@@ -49,7 +53,6 @@ class ExecutorType(type(JAMLCompatible), type):
         :param cls: The class.
         :return: The class, after being registered.
         """
-
         reg_cls_set = getattr(cls, '_registered_class', set())
 
         cls_id = f'{cls.__module__}.{cls.__name__}'
@@ -63,7 +66,9 @@ class ExecutorType(type(JAMLCompatible), type):
                     f'{cls.__init__} does not follow the full signature of `Executor.__init__`, '
                     f'please add `**kwargs` to your __init__ function'
                 )
-            wrap_func(cls, ['__init__'], store_init_kwargs)
+            taboo = get_executor_taboo()
+
+            wrap_func(cls, ['__init__'], store_init_kwargs, taboo=taboo)
             wrap_func(cls, ['__init__'], avoid_concurrent_lock_cls(cls))
 
             reg_cls_set.add(cls_id)
@@ -119,6 +124,7 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
         metas: Optional[Dict] = None,
         requests: Optional[Dict] = None,
         runtime_args: Optional[Dict] = None,
+        workspace: Optional[str] = None,
         **kwargs,
     ):
         """`metas` and `requests` are always auto-filled with values from YAML config.
@@ -127,11 +133,14 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
         :param requests: a dict of endpoint-function mapping
         :param runtime_args: a dict of arguments injected from :class:`Runtime` during runtime
         :param kwargs: additional extra keyword arguments to avoid failing when extra params ara passed that are not expected
+        :param workspace: the workspace of the executor. Only used if a workspace is not already provided in `metas` or `runtime_args`
         """
         self._add_metas(metas)
         self._add_requests(requests)
         self._add_runtime_args(runtime_args)
+        self._init_instrumentation(runtime_args)
         self._init_monitoring()
+        self._init_workspace = workspace
         self.logger = JinaLogger(self.__class__.__name__)
         if __dry_run_endpoint__ not in self.requests:
             self.requests[__dry_run_endpoint__] = self._dry_run_func
@@ -146,12 +155,6 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
     def _dry_run_func(self, *args, **kwargs):
         pass
 
-    def _add_runtime_args(self, _runtime_args: Optional[Dict]):
-        if _runtime_args:
-            self.runtime_args = SimpleNamespace(**_runtime_args)
-        else:
-            self.runtime_args = SimpleNamespace()
-
     def _init_monitoring(self):
         if (
             hasattr(self.runtime_args, 'metrics_registry')
@@ -176,9 +179,59 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
             self._summary_method = None
             self._metrics_buffer = None
 
+        if self.meter:
+            self._process_request_histogram = self.meter.create_histogram(
+                name='jina_process_request_seconds',
+                description='Time spent when calling the executor request method',
+            )
+            self._histogram_buffer = {
+                'jina_process_request_seconds': self._process_request_histogram
+            }
+        else:
+            self._process_request_histogram = None
+            self._histogram_buffer = None
+
+    def _init_instrumentation(self, _runtime_args: Optional[Dict] = None):
+        if not _runtime_args:
+            _runtime_args = {}
+
+        instrumenting_module_name = _runtime_args.get('name', self.__class__.__name__)
+
+        args_tracer_provider = _runtime_args.get('tracer_provider', None)
+        if args_tracer_provider:
+            self.tracer_provider = args_tracer_provider
+            self.tracer = self.tracer_provider.get_tracer(instrumenting_module_name)
+        else:
+            self.tracer_provider = None
+            self.tracer = None
+
+        args_meter_provider = _runtime_args.get('meter_provider', None)
+        if args_meter_provider:
+            self.meter_provider = args_meter_provider
+            self.meter = self.meter_provider.get_meter(instrumenting_module_name)
+        else:
+            self.meter_provider = None
+            self.meter = None
+
+    @property
+    def requests(self):
+        """
+        Get the request dictionary corresponding to this specific class
+
+        :return: Returns the requests corresponding to the specific Executor instance class
+        """
+        if hasattr(self, '_requests'):
+            return self._requests
+        else:
+            if not hasattr(self, 'requests_by_class'):
+                self.requests_by_class = {}
+            if self.__class__.__name__ not in self.requests_by_class:
+                self.requests_by_class[self.__class__.__name__] = {}
+            # we need to copy so that different instances with different (requests) in input do not disturb one another
+            self._requests = copy.copy(self.requests_by_class[self.__class__.__name__])
+            return self._requests
+
     def _add_requests(self, _requests: Optional[Dict]):
-        if not hasattr(self, 'requests'):
-            self.requests = {}
         if _requests:
             func_names = {f.__name__: e for e, f in self.requests.items()}
             for endpoint, func in _requests.items():
@@ -285,12 +338,24 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
         # noqa: DAR102
         # noqa: DAR201
         """
+
         if req_endpoint in self.requests:
             return await self.__acall_endpoint__(req_endpoint, **kwargs)
         elif __default_endpoint__ in self.requests:
             return await self.__acall_endpoint__(__default_endpoint__, **kwargs)
 
-    async def __acall_endpoint__(self, req_endpoint, **kwargs):
+    async def __acall_endpoint__(
+        self, req_endpoint, tracing_context: Optional['Context'], **kwargs
+    ):
+        async def exec_func(
+            summary, histogram, histogram_metric_labels, tracing_context
+        ):
+            with MetricsTimer(summary, histogram, histogram_metric_labels):
+                if iscoroutinefunction(func):
+                    return await func(self, tracing_context=tracing_context, **kwargs)
+                else:
+                    return func(self, tracing_context=tracing_context, **kwargs)
+
         func = self.requests[req_endpoint]
 
         runtime_name = (
@@ -300,16 +365,40 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
         _summary = (
             self._summary_method.labels(
                 self.__class__.__name__, req_endpoint, runtime_name
-            ).time()
+            )
             if self._summary_method
-            else contextlib.nullcontext()
+            else None
         )
+        _histogram_metric_labels = {
+            'executor': self.__class__.__name__,
+            'executor_endpoint': req_endpoint,
+            'runtime_name': runtime_name,
+        }
 
-        with _summary:
-            if iscoroutinefunction(func):
-                return await func(self, **kwargs)
-            else:
-                return func(self, **kwargs)
+        if self.tracer:
+            with self.tracer.start_as_current_span(
+                req_endpoint, context=tracing_context
+            ):
+                from opentelemetry.propagate import extract
+                from opentelemetry.trace.propagation.tracecontext import (
+                    TraceContextTextMapPropagator,
+                )
+
+                tracing_carrier_context = {}
+                TraceContextTextMapPropagator().inject(tracing_carrier_context)
+                return await exec_func(
+                    _summary,
+                    self._process_request_histogram,
+                    _histogram_metric_labels,
+                    extract(tracing_carrier_context),
+                )
+        else:
+            return await exec_func(
+                _summary,
+                self._process_request_histogram,
+                _histogram_metric_labels,
+                None,
+            )
 
     @property
     def workspace(self) -> Optional[str]:
@@ -321,7 +410,8 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
         workspace = (
             getattr(self.runtime_args, 'workspace', None)
             or getattr(self.metas, 'workspace')
-            or os.environ.get('JINA_DEFAULT_WORKSPACE_BASE')
+            or self._init_workspace
+            or __cache_path__
         )
         if workspace:
             complete_workspace = os.path.join(workspace, self.metas.name)
@@ -373,12 +463,12 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
             )
 
         """
-        from jina.hubble.helper import is_valid_huburi
+        from hubble.executor.helper import is_valid_huburi
 
         _source = None
         if is_valid_huburi(uri):
-            from jina.hubble.hubio import HubIO
-            from jina.parsers.hubble import set_hub_pull_parser
+            from hubble.executor.hubio import HubIO
+            from hubble.executor.parsers import set_hub_pull_parser
 
             _args = ArgNamespace.kwargs2namespace(
                 {'no_usage': True, **kwargs},
@@ -523,7 +613,7 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
 
     def monitor(
         self, name: Optional[str] = None, documentation: Optional[str] = None
-    ) -> Optional['Summary']:
+    ) -> Optional[MetricsTimer]:
         """
         Get a given prometheus metric, if it does not exist yet, it will create it and store it in a buffer.
         :param name: the name of the metrics
@@ -531,18 +621,36 @@ class BaseExecutor(JAMLCompatible, metaclass=ExecutorType):
 
         :return: the given prometheus metrics or None if monitoring is not enable.
         """
+        _summary = (
+            self._metrics_buffer.get(name, None) if self._metrics_buffer else None
+        )
+        _histogram = (
+            self._histogram_buffer.get(name, None) if self._histogram_buffer else None
+        )
 
-        if self._metrics_buffer:
-            if name not in self._metrics_buffer:
-                from prometheus_client import Summary
+        if self._metrics_buffer and not _summary:
+            from prometheus_client import Summary
 
-                self._metrics_buffer[name] = Summary(
-                    name,
-                    documentation,
-                    registry=self.runtime_args.metrics_registry,
-                    namespace='jina',
-                    labelnames=('runtime_name',),
-                ).labels(self.runtime_args.name)
-            return self._metrics_buffer[name].time()
-        else:
-            return contextlib.nullcontext()
+            _summary = Summary(
+                name,
+                documentation,
+                registry=self.runtime_args.metrics_registry,
+                namespace='jina',
+                labelnames=('runtime_name',),
+            ).labels(self.runtime_args.name)
+            self._metrics_buffer[name] = _summary
+
+        if self._histogram_buffer and not _histogram:
+            _histogram = self.meter.create_histogram(
+                name=f'jina_{name}', description=documentation
+            )
+            self._histogram_buffer[name] = _histogram
+
+        if _summary or _histogram:
+            return MetricsTimer(
+                _summary,
+                _histogram,
+                histogram_metric_labels={'runtime_name': self.runtime_args.name},
+            )
+
+        return contextlib.nullcontext()
