@@ -2,24 +2,20 @@
 import functools
 import inspect
 import os
-from contextlib import nullcontext
 from pathlib import Path
 from typing import TYPE_CHECKING, Callable, Dict, List, Optional, Sequence, Union
 
 from jina import __cache_path__
-from jina.helper import convert_tuple_to_list, iscoroutinefunction
+from jina.helper import iscoroutinefunction
 from jina.importer import ImportExtensions
-from jina.serve.executors.metas import get_default_metas
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from jina import DocumentArray
 
 
 @functools.lru_cache()
 def _get_locks_root() -> Path:
-    locks_root = Path(
-        os.environ.get('JINA_LOCKS_ROOT', os.path.join(__cache_path__, 'locks'))
-    )
+    locks_root = Path(os.path.join(__cache_path__, 'locks'))
 
     if not locks_root.exists():
         locks_root.mkdir(parents=True, exist_ok=True)
@@ -27,57 +23,6 @@ def _get_locks_root() -> Path:
     return locks_root
 
 
-def wrap_func(cls, func_lst, wrapper):
-    """Wrapping a class method only once, inherited but not overridden method will not be wrapped again
-
-    :param cls: class
-    :param func_lst: function list to wrap
-    :param wrapper: the wrapper
-    """
-    for f_name in func_lst:
-        if hasattr(cls, f_name) and all(
-            getattr(cls, f_name) != getattr(i, f_name, None) for i in cls.mro()[1:]
-        ):
-            setattr(cls, f_name, wrapper(getattr(cls, f_name)))
-
-
-def store_init_kwargs(func: Callable) -> Callable:
-    """Mark the args and kwargs of :func:`__init__` later to be stored via :func:`save_config` in YAML
-    :param func: the function to decorate
-    :return: the wrapped function
-    """
-
-    @functools.wraps(func)
-    def arg_wrapper(self, *args, **kwargs):
-        if func.__name__ != '__init__':
-            raise TypeError(
-                'this decorator should only be used on __init__ method of an executor'
-            )
-        taboo = {'self', 'args', 'kwargs', 'metas', 'requests', 'runtime_args'}
-        _defaults = get_default_metas()
-        taboo.update(_defaults.keys())
-        all_pars = inspect.signature(func).parameters
-        tmp = {k: v.default for k, v in all_pars.items() if k not in taboo}
-        tmp_list = [k for k in all_pars.keys() if k not in taboo]
-        # set args by aligning tmp_list with arg values
-        for k, v in zip(tmp_list, args):
-            tmp[k] = v
-        # set kwargs
-        for k, v in kwargs.items():
-            if k in tmp:
-                tmp[k] = v
-
-        if hasattr(self, '_init_kwargs_dict'):
-            self._init_kwargs_dict.update(tmp)
-        else:
-            self._init_kwargs_dict = tmp
-        convert_tuple_to_list(self._init_kwargs_dict)
-        f = func(self, *args, **kwargs)
-        return f
-
-    return arg_wrapper
-
-
 def avoid_concurrent_lock_cls(cls):
     """Wraps a function to lock a filelock for concurrent access with the name of the class to which it applies, to avoid deadlocks
     :param cls: the class to which is applied, only when the class corresponds to the instance type, this filelock will apply
@@ -98,11 +43,10 @@ def avoid_concurrent_lock_cls(cls):
                 )
 
             if self.__class__ == cls:
-                file_lock = nullcontext()
                 with ImportExtensions(
-                    required=False,
-                    help_text=f'FileLock is needed to guarantee non-concurrent initialization of replicas in the same '
-                    f'machine.',
+                        required=False,
+                        help_text=f'FileLock is needed to guarantee non-concurrent initialization of replicas in the '
+                                  f'same machine.',
                 ):
                     import filelock
 
@@ -123,19 +67,44 @@ def avoid_concurrent_lock_cls(cls):
     return avoid_concurrent_lock_wrapper
 
 
+def _init_requests_by_class(cls):
+    """
+    To allow inheritance and still have coherent usage of `requests`. Makes sure that a child class inherits requests from parents
+
+    :param cls: The class.
+    """
+    if not hasattr(cls, 'requests_by_class'):
+        cls.requests_by_class = {}
+
+    if cls.__name__ not in cls.requests_by_class:
+        cls.requests_by_class[cls.__name__] = {}
+
+        def _inherit_from_parent_class_inner(cls_):
+            for parent_class in cls_.__bases__:
+                parent_dict = cls.requests_by_class.get(parent_class.__name__, {})
+                for k, v in parent_dict.items():
+                    if k not in cls.requests_by_class[cls.__name__]:
+                        cls.requests_by_class[cls.__name__][k] = v
+                _inherit_from_parent_class_inner(parent_class)
+
+        # assume that `requests` is called when importing class, so parent classes will be processed before
+        # inherit all the requests from parents
+        _inherit_from_parent_class_inner(cls)
+
+
 def requests(
-    func: Callable[
-        [
-            'DocumentArray',
-            Dict,
-            'DocumentArray',
-            List['DocumentArray'],
-            List['DocumentArray'],
-        ],
-        Optional[Union['DocumentArray', Dict]],
-    ] = None,
-    *,
-    on: Optional[Union[str, Sequence[str]]] = None,
+        func: Optional[Callable[
+            [
+                'DocumentArray',
+                Dict,
+                'DocumentArray',
+                List['DocumentArray'],
+                List['DocumentArray'],
+            ],
+            Optional[Union['DocumentArray', Dict]],
+        ]] = None,
+        *,
+        on: Optional[Union[str, Sequence[str]]] = None,
 ):
     """
     `@requests` defines the endpoints of an Executor. It has a keyword `on=` to define the endpoint.
@@ -187,10 +156,9 @@ def requests(
 
     class FunctionMapper:
         def __init__(self, fn):
-
             arg_spec = inspect.getfullargspec(fn)
             if not arg_spec.varkw and not __args_executor_func__.issubset(
-                arg_spec.args
+                    arg_spec.args
             ):
                 raise TypeError(
                     f'{fn} accepts only {arg_spec.args} which is fewer than expected, '
@@ -198,10 +166,9 @@ def requests(
                 )
 
             if iscoroutinefunction(fn):
-
                 @functools.wraps(fn)
                 async def arg_wrapper(
-                    executor_instance, *args, **kwargs
+                        executor_instance, *args, **kwargs
                 ):  # we need to get the summary from the executor, so we need to access the self
                     return await fn(executor_instance, *args, **kwargs)
 
@@ -210,22 +177,20 @@ def requests(
 
                 @functools.wraps(fn)
                 def arg_wrapper(
-                    executor_instance, *args, **kwargs
+                        executor_instance, *args, **kwargs
                 ):  # we need to get the summary from the executor, so we need to access the self
                     return fn(executor_instance, *args, **kwargs)
 
                 self.fn = arg_wrapper
 
         def __set_name__(self, owner, name):
+            _init_requests_by_class(owner)
             self.fn.class_name = owner.__name__
-            if not hasattr(owner, 'requests'):
-                owner.requests = {}
-
             if isinstance(on, (list, tuple)):
                 for o in on:
-                    owner.requests[o] = self.fn
+                    owner.requests_by_class[owner.__name__][o] = self.fn
             else:
-                owner.requests[on or __default_endpoint__] = self.fn
+                owner.requests_by_class[owner.__name__][on or __default_endpoint__] = self.fn
 
             setattr(owner, name, self.fn)
 
@@ -236,9 +201,9 @@ def requests(
 
 
 def monitor(
-    *,
-    name: Optional[str] = None,
-    documentation: Optional[str] = None,
+        *,
+        name: Optional[str] = None,
+        documentation: Optional[str] = None,
 ):
     """
     Decorator and context manager that allows monitoring of an Executor.
@@ -304,7 +269,6 @@ def monitor(
     """
 
     def _decorator(func: Callable):
-
         name_ = name if name else f'{func.__name__}_seconds'
         documentation_ = (
             documentation
