@@ -11,7 +11,7 @@ from jina import __default_endpoint__
 from jina.excepts import InternalNetworkError
 from jina.serve.networking import GrpcConnectionPool
 from jina.serve.runtimes.helper import _parse_specific_params
-from jina.serve.runtimes.request_handlers.data_request_handler import DataRequestHandler
+from jina.serve.runtimes.worker.request_handling import WorkerRequestHandler
 from jina.types.request.data import DataRequest
 
 
@@ -22,7 +22,7 @@ class TopologyGraph:
 
     :param graph_description: A dictionary describing the topology of the Deployments. 2 special nodes are expected, the name `start-gateway` and `end-gateway` to
         determine the nodes that receive the very first request and the ones whose response needs to be sent back to the client. All the nodes with no outgoing nodes
-        will be considered to be hanging, and they will be "flagged" so that the user can ignore their tasks and not await them.
+        will be considered to be floating, and they will be "flagged" so that the user can ignore their tasks and not await them.
 
     :param conditions: A dictionary describing which Executors have special conditions to be fullfilled by the `Documents` to be sent to them.
     :param reduce: Reduce requests arriving from multiple needed predecessors, True by default
@@ -33,8 +33,9 @@ class TopologyGraph:
             self,
             name: str,
             number_of_parts: int = 1,
-            hanging: bool = False,
+            floating: bool = False,
             filter_condition: dict = None,
+            metadata: Optional[Dict] = None,
             reduce: bool = True,
             timeout_send: Optional[float] = None,
             retries: Optional[int] = -1,
@@ -42,35 +43,40 @@ class TopologyGraph:
             self.name = name
             self.outgoing_nodes = []
             self.number_of_parts = number_of_parts
-            self.hanging = hanging
+            self.floating = floating
             self.parts_to_send = []
-            self.original_parameters = []
             self.start_time = None
             self.end_time = None
             self.status = None
             self._filter_condition = filter_condition
+            self._metadata = metadata
             self._reduce = reduce
             self._timeout_send = timeout_send
             self._retries = retries
+            self.result_in_params_returned = None
 
         @property
         def leaf(self):
             return len(self.outgoing_nodes) == 0
 
-        def _update_requests_with_filter_condition(self):
+        def _update_requests_with_filter_condition(self, need_copy):
             for i in range(len(self.parts_to_send)):
-                copy_req = copy.deepcopy(self.parts_to_send[i])
-                filtered_docs = copy_req.docs.find(self._filter_condition)
-                copy_req.data.docs = filtered_docs
-
-                self.parts_to_send[i] = copy_req
-
-        def _update_request_by_params(self, deployment_name: str):
-
-            for i in range(len(self.parts_to_send)):
-                specific_parameters = _parse_specific_params(
-                    self.original_parameters[i], deployment_name
+                req = (
+                    self.parts_to_send[i]
+                    if not need_copy
+                    else copy.deepcopy(self.parts_to_send[i])
                 )
+                filtered_docs = req.docs.find(self._filter_condition)
+                req.data.docs = filtered_docs
+                self.parts_to_send[i] = req
+
+        def _update_request_by_params(
+            self, deployment_name: str, request_input_parameters: Dict
+        ):
+            specific_parameters = _parse_specific_params(
+                request_input_parameters, deployment_name
+            )
+            for i in range(len(self.parts_to_send)):
                 self.parts_to_send[i].parameters = specific_parameters
 
         def _handle_internalnetworkerror(self, err):
@@ -93,16 +99,20 @@ class TopologyGraph:
                 raise
 
         def get_endpoints(self, connection_pool: GrpcConnectionPool) -> asyncio.Task:
-            return connection_pool.send_discover_endpoint(self.name)
+            return connection_pool.send_discover_endpoint(
+                self.name, retries=self._retries
+            )
 
         async def _wait_previous_and_send(
             self,
-            request: DataRequest,
+            request: Optional[DataRequest],
             previous_task: Optional[asyncio.Task],
             connection_pool: GrpcConnectionPool,
             endpoint: Optional[str],
             executor_endpoint_mapping: Optional[Dict] = None,
             target_executor_pattern: Optional[str] = None,
+            request_input_parameters: Dict = {},
+            copy_request_at_send: bool = False,
         ):
             # Check my condition and send request with the condition
             metadata = {}
@@ -112,21 +122,24 @@ class TopologyGraph:
             if metadata and 'is-error' in metadata:
                 return request, metadata
             elif request is not None:
-                original_parameters = copy.deepcopy(request.parameters)
                 request.parameters = _parse_specific_params(
                     request.parameters, self.name
                 )
-                self.parts_to_send.append(request)
-                self.original_parameters.append(original_parameters)
+                req_to_send = (
+                    copy.deepcopy(request) if copy_request_at_send else request
+                )
+                self.parts_to_send.append(req_to_send)
                 # this is a specific needs
                 if len(self.parts_to_send) == self.number_of_parts:
                     self.start_time = datetime.utcnow()
-                    self._update_request_by_params(self.name)
+                    self._update_request_by_params(self.name, request_input_parameters)
                     if self._filter_condition is not None:
-                        self._update_requests_with_filter_condition()
+                        self._update_requests_with_filter_condition(
+                            need_copy=not copy_request_at_send
+                        )
                     if self._reduce and len(self.parts_to_send) > 1:
                         self.parts_to_send = [
-                            DataRequestHandler.reduce_requests(self.parts_to_send)
+                            WorkerRequestHandler.reduce_requests(self.parts_to_send)
                         ]
 
                     # avoid sending to executor which does not bind to this endpoint
@@ -144,21 +157,27 @@ class TopologyGraph:
                         return request, metadata
                     # otherwise, send to executor and get response
                     try:
-                        resp, metadata = await connection_pool.send_requests_once(
+                        result = await connection_pool.send_requests_once(
                             requests=self.parts_to_send,
                             deployment=self.name,
+                            metadata=self._metadata,
                             head=True,
                             endpoint=endpoint,
                             timeout=self._timeout_send,
                             retries=self._retries,
                         )
-                        return_parameters = original_parameters
-                        if DataRequestHandler._KEY_RESULT in resp.parameters:
-                            return_parameters[
-                                DataRequestHandler._KEY_RESULT
-                            ] = resp.parameters[DataRequestHandler._KEY_RESULT]
-
-                        resp.parameters = return_parameters
+                        if issubclass(type(result), BaseException):
+                            raise result
+                        else:
+                            resp, metadata = result
+                        if WorkerRequestHandler._KEY_RESULT in resp.parameters:
+                            # Accumulate results from each Node and then add them to the original
+                            self.result_in_params_returned = resp.parameters[
+                                WorkerRequestHandler._KEY_RESULT
+                            ]
+                        request.parameters = request_input_parameters
+                        resp.parameters = request_input_parameters
+                        self.parts_to_send.clear()
                     except InternalNetworkError as err:
                         self._handle_internalnetworkerror(err)
 
@@ -177,6 +196,9 @@ class TopologyGraph:
             endpoint: Optional[str] = None,
             executor_endpoint_mapping: Optional[Dict] = None,
             target_executor_pattern: Optional[str] = None,
+            request_input_parameters: Dict = {},
+            request_input_has_specific_params: bool = False,
+            copy_request_at_send: bool = False,
         ) -> List[Tuple[bool, asyncio.Task]]:
             """
             Gets all the tasks corresponding from all the subgraphs born from this node
@@ -187,6 +209,9 @@ class TopologyGraph:
             :param endpoint: Optional string defining the endpoint of this request
             :param executor_endpoint_mapping: Optional map that maps the name of a Deployment with the endpoints that it binds to so that they can be skipped if needed
             :param target_executor_pattern: Optional regex pattern for the target executor to decide whether or not the Executor should receive the request
+            :param request_input_parameters: The parameters coming from the Request as they arrive to the gateway
+            :param request_input_has_specific_params: Parameter added for optimization. If this is False, there is no need to copy at all the request
+            :param copy_request_at_send: Copy the request before actually calling the `ConnectionPool` sending
 
             .. note:
                 deployment1 -> outgoing_nodes: deployment2
@@ -212,27 +237,34 @@ class TopologyGraph:
             """
             wait_previous_and_send_task = asyncio.create_task(
                 self._wait_previous_and_send(
-                    request_to_send,
-                    previous_task,
-                    connection_pool,
+                    request=request_to_send,
+                    previous_task=previous_task,
+                    connection_pool=connection_pool,
                     endpoint=endpoint,
                     executor_endpoint_mapping=executor_endpoint_mapping,
                     target_executor_pattern=target_executor_pattern,
+                    request_input_parameters=request_input_parameters,
+                    copy_request_at_send=copy_request_at_send,
                 )
             )
             if self.leaf:  # I am like a leaf
                 return [
-                    (not self.hanging, wait_previous_and_send_task)
+                    (not self.floating, wait_previous_and_send_task)
                 ]  # I am the last in the chain
             hanging_tasks_tuples = []
+            num_outgoing_nodes = len(self.outgoing_nodes)
             for outgoing_node in self.outgoing_nodes:
                 t = outgoing_node.get_leaf_tasks(
-                    connection_pool,
-                    None,
-                    wait_previous_and_send_task,
+                    connection_pool=connection_pool,
+                    request_to_send=None,
+                    previous_task=wait_previous_and_send_task,
                     endpoint=endpoint,
                     executor_endpoint_mapping=executor_endpoint_mapping,
                     target_executor_pattern=target_executor_pattern,
+                    request_input_parameters=request_input_parameters,
+                    request_input_has_specific_params=request_input_has_specific_params,
+                    copy_request_at_send=num_outgoing_nodes > 1
+                    and request_input_has_specific_params,
                 )
                 # We are interested in the last one, that will be the task that awaits all the previous
                 hanging_tasks_tuples.extend(t)
@@ -266,11 +298,35 @@ class TopologyGraph:
                 request = outgoing_node.add_route(request=request)
             return request
 
+    class _EndGatewayNode(_ReqReplyNode):
+
+        """
+        Dummy node to be added before the gateway. This is to solve a problem we had when implementing `floating Executors`.
+        If we do not add this at the end, this structure does not work:
+
+            GATEWAY -> EXEC1 -> FLOATING
+                    -> GATEWAY
+        """
+
+        def get_endpoints(self, *args, **kwargs) -> asyncio.Task:
+            async def task_wrapper():
+                from jina.serve.networking import default_endpoints_proto
+
+                return default_endpoints_proto, None
+
+            return asyncio.create_task(task_wrapper())
+
+        def get_leaf_tasks(
+            self, previous_task: Optional[asyncio.Task], *args, **kwargs
+        ) -> List[Tuple[bool, asyncio.Task]]:
+            return [(True, previous_task)]
+
     def __init__(
         self,
         graph_representation: Dict,
         graph_conditions: Dict = {},
-        deployments_disable_reduce: List[str] = [],
+        deployments_metadata: Dict = {},
+        deployments_no_reduce: List[str] = [],
         timeout_send: Optional[float] = 1.0,
         retries: Optional[int] = -1,
         *args,
@@ -281,13 +337,13 @@ class TopologyGraph:
             origin_node_names = graph_representation['start-gateway']
         else:
             origin_node_names = set()
-        hanging_deployment_names = set()
+        floating_deployment_set = set()
         node_set = set()
         for node_name, outgoing_node_names in graph_representation.items():
             if node_name not in {'start-gateway', 'end-gateway'}:
                 node_set.add(node_name)
             if len(outgoing_node_names) == 0:
-                hanging_deployment_names.add(node_name)
+                floating_deployment_set.add(node_name)
             for out_node_name in outgoing_node_names:
                 if out_node_name not in {'start-gateway', 'end-gateway'}:
                     node_set.add(out_node_name)
@@ -296,14 +352,16 @@ class TopologyGraph:
         nodes = {}
         for node_name in node_set:
             condition = graph_conditions.get(node_name, None)
+            metadata = deployments_metadata.get(node_name, None)
             nodes[node_name] = self._ReqReplyNode(
                 name=node_name,
                 number_of_parts=num_parts_per_node[node_name]
                 if num_parts_per_node[node_name] > 0
                 else 1,
-                hanging=node_name in hanging_deployment_names,
+                floating=node_name in floating_deployment_set,
                 filter_condition=condition,
-                reduce=node_name not in deployments_disable_reduce,
+                metadata=metadata,
+                reduce=node_name not in deployments_no_reduce,
                 timeout_send=timeout_send,
                 retries=retries,
             )
@@ -313,6 +371,10 @@ class TopologyGraph:
                 for out_node_name in outgoing_node_names:
                     if out_node_name not in ['start-gateway', 'end-gateway']:
                         nodes[node_name].outgoing_nodes.append(nodes[out_node_name])
+                    if out_node_name == 'end-gateway':
+                        nodes[node_name].outgoing_nodes.append(
+                            self._EndGatewayNode(name='__end_gateway__', floating=False)
+                        )
 
         self._origin_nodes = [nodes[node_name] for node_name in origin_node_names]
         self.has_filter_conditions = bool(graph_conditions)
@@ -363,3 +425,14 @@ class TopologyGraph:
                     nodes.append(st_node)
                     node_names.append(st_node_name)
         return nodes
+
+    def collect_all_results(self):
+        """Collect all the results from every node into a single dictionary so that gateway can collect them
+
+        :return: A dictionary of the results
+        """
+        res = {}
+        for node in self.all_nodes:
+            if node.result_in_params_returned:
+                res.update(node.result_in_params_returned)
+        return res
