@@ -1,27 +1,28 @@
 import asyncio
 import copy
-import time
 from typing import TYPE_CHECKING, Callable, List, Optional, Tuple
 
 import grpc.aio
 
 from docarray import DocumentArray
 from jina.excepts import InternalNetworkError
-from jina.importer import ImportExtensions
+from jina.helper import GATEWAY_NAME
 from jina.serve.networking import GrpcConnectionPool
 from jina.serve.runtimes.gateway.graph.topology_graph import TopologyGraph
 from jina.serve.runtimes.helper import _is_param_for_specific_executor
-from jina.serve.runtimes.request_handlers.data_request_handler import DataRequestHandler
+from jina.serve.runtimes.monitoring import MonitoringRequestMixin
+from jina.serve.runtimes.worker.request_handling import WorkerRequestHandler
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from asyncio import Future
 
+    from opentelemetry.metrics import Meter
     from prometheus_client import CollectorRegistry
 
     from jina.types.request import Request
 
 
-class RequestHandler:
+class GatewayRequestHandler(MonitoringRequestMixin):
     """
     Class that handles the requests arriving to the gateway and the result extracted from the requests future.
 
@@ -30,59 +31,17 @@ class RequestHandler:
     """
 
     def __init__(
-            self,
-            metrics_registry: Optional['CollectorRegistry'] = None,
-            runtime_name: Optional[str] = None,
+        self,
+        metrics_registry: Optional['CollectorRegistry'] = None,
+        meter: Optional['Meter'] = None,
+        runtime_name: Optional[str] = None,
     ):
-        self._request_init_time = {} if metrics_registry else None
+        super().__init__(metrics_registry, meter, runtime_name)
         self._executor_endpoint_mapping = None
         self._gathering_endpoints = False
 
-        if metrics_registry:
-            with ImportExtensions(
-                    required=True,
-                    help_text='You need to install the `prometheus_client` to use the montitoring functionality of jina',
-            ):
-                from prometheus_client import Gauge, Summary
-
-            self._receiving_request_metrics = Summary(
-                'receiving_request_seconds',
-                'Time spent processing request',
-                registry=metrics_registry,
-                namespace='jina',
-                labelnames=('runtime_name',),
-            ).labels(runtime_name)
-
-            self._pending_requests_metrics = Gauge(
-                'number_of_pending_requests',
-                'Number of pending requests',
-                registry=metrics_registry,
-                namespace='jina',
-                labelnames=('runtime_name',),
-            ).labels(runtime_name)
-
-        else:
-            self._receiving_request_metrics = None
-            self._pending_requests_metrics = None
-
-    def _update_start_request_metrics(self, request: 'Request'):
-        if self._receiving_request_metrics:
-            self._request_init_time[request.request_id] = time.time()
-        if self._pending_requests_metrics:
-            self._pending_requests_metrics.inc()
-
-    def _update_end_request_metrics(self, result: 'Request'):
-        if self._receiving_request_metrics:
-            init_time = self._request_init_time.pop(
-                result.request_id
-            )  # need to pop otherwise it stays in memory forever
-            self._receiving_request_metrics.observe(time.time() - init_time)
-
-        if self._pending_requests_metrics:
-            self._pending_requests_metrics.dec()
-
     def handle_request(
-            self, graph: 'TopologyGraph', connection_pool: 'GrpcConnectionPool'
+        self, graph: 'TopologyGraph', connection_pool: 'GrpcConnectionPool'
     ) -> Callable[['Request'], 'Tuple[Future, Optional[Future]]']:
         """
         Function that handles the requests arriving to the gateway. This will be passed to the streamer.
@@ -103,8 +62,8 @@ class RequestHandler:
                 err_code = err.code()
                 if err_code == grpc.StatusCode.UNAVAILABLE:
                     err._details = (
-                            err.details()
-                            + f' |Gateway: Communication error with deployment at address(es) {err.dest_addr}. Head or worker(s) may be down.'
+                        err.details()
+                        + f' |Gateway: Communication error with deployment at address(es) {err.dest_addr}. Head or worker(s) may be down.'
                     )
                     raise err
                 else:
@@ -122,8 +81,8 @@ class RequestHandler:
 
             if graph.has_filter_conditions:
                 request_doc_ids = request.data.docs[
-                                  :, 'id'
-                                  ]  # used to maintain order of docs that are filtered by executors
+                    :, 'id'
+                ]  # used to maintain order of docs that are filtered by executors
             responding_tasks = []
             floating_tasks = []
             endpoint = request.header.exec_endpoint
@@ -154,7 +113,7 @@ class RequestHandler:
                     target_executor_pattern=target_executor or None,
                     request_input_parameters=request_input_parameters,
                     request_input_has_specific_params=has_specific_params,
-                    copy_request_at_send=num_outgoing_nodes > 1 and has_specific_params
+                    copy_request_at_send=num_outgoing_nodes > 1 and has_specific_params,
                 )
                 # Every origin node returns a set of tasks that are the ones corresponding to the leafs of each of their
                 # subtrees that unwrap all the previous tasks. It starts like a chain of waiting for tasks from previous
@@ -174,19 +133,20 @@ class RequestHandler:
                 response.data.docs = DocumentArray(sorted_docs)
 
             async def _process_results_at_end_gateway(
-                    tasks: List[asyncio.Task], request_graph: TopologyGraph
+                tasks: List[asyncio.Task], request_graph: TopologyGraph
             ) -> asyncio.Future:
                 try:
                     if (
-                            self._executor_endpoint_mapping is None
-                            and not self._gathering_endpoints
+                        self._executor_endpoint_mapping is None
+                        and not self._gathering_endpoints
                     ):
                         self._gathering_endpoints = True
                         asyncio.create_task(gather_endpoints(request_graph))
 
                     partial_responses = await asyncio.gather(*tasks)
-                except:
-                    self._update_end_request_metrics(request)
+                except Exception:
+                    # update here failed request
+                    self._update_end_failed_requests_metrics()
                     raise
                 partial_responses, metadatas = zip(*partial_responses)
                 filtered_partial_responses = list(
@@ -194,6 +154,12 @@ class RequestHandler:
                 )
 
                 response = filtered_partial_responses[0]
+                # JoanFM: to keep the docs_map feature, need to add the routes in the WorkerRuntime but clear it here
+                # so that routes are properly done. not very clean but refactoring would be costly for such a small
+                # thing, `docs_map` reuses routes potentially not in the best way but works for now
+                for i in reversed(range(len(response.routes))):
+                    if response.routes[i].executor != GATEWAY_NAME:
+                        del response.routes[i]
                 request_graph.add_routes(response)
 
                 if graph.has_filter_conditions:
@@ -202,7 +168,7 @@ class RequestHandler:
                 collect_results = request_graph.collect_all_results()
                 resp_params = response.parameters
                 if len(collect_results) > 0:
-                    resp_params[DataRequestHandler._KEY_RESULT] = collect_results
+                    resp_params[WorkerRequestHandler._KEY_RESULT] = collect_results
                     response.parameters = resp_params
                 return response
 
