@@ -1,77 +1,121 @@
 import argparse
-import contextlib
 from abc import ABC
-from typing import List
+from typing import TYPE_CHECKING, List, Optional
 
 import grpc
 from grpc_health.v1 import health, health_pb2, health_pb2_grpc
 from grpc_reflection.v1alpha import reflection
 
+from jina.excepts import RuntimeTerminated
 from jina.helper import get_full_version
 from jina.importer import ImportExtensions
 from jina.proto import jina_pb2, jina_pb2_grpc
+from jina.serve.instrumentation import MetricsTimer
 from jina.serve.runtimes.asyncio import AsyncNewLoopRuntime
-from jina.serve.runtimes.request_handlers.data_request_handler import DataRequestHandler
+from jina.serve.runtimes.helper import _get_grpc_server_options
+from jina.serve.runtimes.worker.request_handling import WorkerRequestHandler
 from jina.types.request.data import DataRequest
 
+if TYPE_CHECKING:  # pragma: no cover
+    from opentelemetry.propagate import Context
+
 
 class WorkerRuntime(AsyncNewLoopRuntime, ABC):
     """Runtime procedure leveraging :class:`Grpclet` for sending DataRequests"""
 
     def __init__(
-            self,
-            args: argparse.Namespace,
-            **kwargs,
+        self,
+        args: argparse.Namespace,
+        **kwargs,
     ):
         """Initialize grpc and data request handling.
         :param args: args from CLI
         :param kwargs: keyword args
         """
-        self._health_servicer = health.HealthServicer(experimental_non_blocking=True)
+        self._health_servicer = health.aio.HealthServicer()
         super().__init__(args, **kwargs)
 
     async def async_setup(self):
         """
-        Start the DataRequestHandler and wait for the GRPC and Monitoring servers to start
+        Start the WorkerRequestHandler and wait for the GRPC and Monitoring servers to start
         """
         if self.metrics_registry:
             with ImportExtensions(
-                    required=True,
-                    help_text='You need to install the `prometheus_client` to use the montitoring functionality of jina',
+                required=True,
+                help_text='You need to install the `prometheus_client` to use the montitoring functionality of jina',
             ):
-                from prometheus_client import Summary
+                from prometheus_client import Counter, Summary
 
-            self._summary_time = (
-                Summary(
-                    'receiving_request_seconds',
-                    'Time spent processing request',
-                    registry=self.metrics_registry,
-                    namespace='jina',
-                    labelnames=('runtime_name',),
-                )
-                    .labels(self.args.name)
-                    .time()
+            self._summary = Summary(
+                'receiving_request_seconds',
+                'Time spent processing request',
+                registry=self.metrics_registry,
+                namespace='jina',
+                labelnames=('runtime_name',),
+            ).labels(self.args.name)
+
+            self._failed_requests_metrics = Counter(
+                'failed_requests',
+                'Number of failed requests',
+                registry=self.metrics_registry,
+                namespace='jina',
+                labelnames=('runtime_name',),
+            ).labels(self.args.name)
+
+            self._successful_requests_metrics = Counter(
+                'successful_requests',
+                'Number of successful requests',
+                registry=self.metrics_registry,
+                namespace='jina',
+                labelnames=('runtime_name',),
+            ).labels(self.args.name)
+
+        else:
+            self._summary = None
+            self._failed_requests_metrics = None
+            self._successful_requests_metrics = None
+
+        if self.meter:
+            self._receiving_request_seconds = self.meter.create_histogram(
+                name='jina_receiving_request_seconds',
+                description='Time spent processing request',
+            )
+            self._failed_requests_counter = self.meter.create_counter(
+                name='jina_failed_requests',
+                description='Number of failed requests',
+            )
+
+            self._successful_requests_counter = self.meter.create_counter(
+                name='jina_successful_requests',
+                description='Number of successful requests',
             )
         else:
-            self._summary_time = contextlib.nullcontext()
+            self._receiving_request_seconds = None
+            self._failed_requests_counter = None
+            self._successful_requests_counter = None
+        self._metric_attributes = {'runtime_name': self.args.name}
+
         # Keep this initialization order
         # otherwise readiness check is not valid
-        # The DataRequestHandler needs to be started BEFORE the grpc server
-        self._data_request_handler = DataRequestHandler(
-            self.args, self.logger, self.metrics_registry
+        # The WorkerRequestHandler needs to be started BEFORE the grpc server
+        self._request_handler = WorkerRequestHandler(
+            args=self.args,
+            logger=self.logger,
+            metrics_registry=self.metrics_registry,
+            tracer_provider=self.tracer_provider,
+            meter_provider=self.meter_provider,
+            deployment_name=self.name.split('/')[0],
         )
         await self._async_setup_grpc_server()
 
     async def _async_setup_grpc_server(self):
         """
-        Start the DataRequestHandler and wait for the GRPC server to start
+        Start the WorkerRequestHandler and wait for the GRPC server to start
         """
 
         self._grpc_server = grpc.aio.server(
-            options=[
-                ('grpc.max_send_message_length', -1),
-                ('grpc.max_receive_message_length', -1),
-            ]
+            options=_get_grpc_server_options(self.args.grpc_server_options),
+            interceptors=self.aio_tracing_server_interceptors(),
         )
 
         jina_pb2_grpc.add_JinaSingleDataRequestRPCServicer_to_server(
@@ -96,7 +140,9 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
         )
 
         for service in service_names:
-            self._health_servicer.set(service, health_pb2.HealthCheckResponse.SERVING)
+            await self._health_servicer.set(
+                service, health_pb2.HealthCheckResponse.SERVING
+            )
         reflection.enable_server_reflection(service_names, self._grpc_server)
         bind_addr = f'0.0.0.0:{self.args.port}'
         self.logger.debug(f'start listening on {bind_addr}')
@@ -118,9 +164,9 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
 
     async def async_teardown(self):
         """Close the data request handler"""
-        self._health_servicer.enter_graceful_shutdown()
+        await self._health_servicer.enter_graceful_shutdown()
         await self.async_cancel()
-        self._data_request_handler.close()
+        self._request_handler.close()
 
     async def process_single_data(self, request: DataRequest, context) -> DataRequest:
         """
@@ -143,10 +189,21 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
         self.logger.debug('got an endpoint discovery request')
         endpointsProto = jina_pb2.EndpointsProto()
         endpointsProto.endpoints.extend(
-            list(self._data_request_handler._executor.requests.keys())
+            list(self._request_handler._executor.requests.keys())
         )
         return endpointsProto
 
+    def _extract_tracing_context(
+        self, metadata: grpc.aio.Metadata
+    ) -> Optional['Context']:
+        if self.tracer:
+            from opentelemetry.propagate import extract
+
+            context = extract(dict(metadata))
+            return context
+
+        return None
+
     async def process_data(self, requests: List[DataRequest], context) -> DataRequest:
         """
         Process the received requests and return the result as a new request
@@ -156,12 +213,26 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
         :returns: the response request
         """
 
-        with self._summary_time:
+        with MetricsTimer(
+            self._summary, self._receiving_request_seconds, self._metric_attributes
+        ):
             try:
                 if self.logger.debug_enabled:
                     self._log_data_request(requests[0])
 
-                return await self._data_request_handler.handle(requests=requests)
+                tracing_context = self._extract_tracing_context(
+                    context.invocation_metadata()
+                )
+                result = await self._request_handler.handle(
+                    requests=requests, tracing_context=tracing_context
+                )
+                if self._successful_requests_metrics:
+                    self._successful_requests_metrics.inc()
+                if self._successful_requests_counter:
+                    self._successful_requests_counter.add(
+                        1, attributes=self._metric_attributes
+                    )
+                return result
             except (RuntimeError, Exception) as ex:
                 self.logger.error(
                     f'{ex!r}'
@@ -171,8 +242,22 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
                     exc_info=not self.args.quiet_error,
                 )
 
-                requests[0].add_exception(ex, self._data_request_handler._executor)
+                requests[0].add_exception(ex, self._request_handler._executor)
                 context.set_trailing_metadata((('is-error', 'true'),))
+                if self._failed_requests_metrics:
+                    self._failed_requests_metrics.inc()
+                if self._failed_requests_counter:
+                    self._failed_requests_counter.add(
+                        1, attributes=self._metric_attributes
+                    )
+
+                if (
+                    self.args.exit_on_exceptions
+                    and type(ex).__name__ in self.args.exit_on_exceptions
+                ):
+                    self.logger.info('Exiting because of "--exit-on-exceptions".')
+                    raise RuntimeTerminated
+
                 return requests[0]
 
     async def _status(self, empty, context) -> jina_pb2.JinaInfoProto:
