@@ -54,7 +54,7 @@ class DataRequest(Request):
             self.set_docs_convert_arrays(value, None)
 
         def set_docs_convert_arrays(
-            self, value: DocumentArray, ndarray_type: Optional[str] = None
+                self, value: DocumentArray, ndarray_type: Optional[str] = None
         ):
             """ " Convert embedding and tensor to given type, then set DocumentArray
 
@@ -100,8 +100,8 @@ class DataRequest(Request):
     """
 
     def __init__(
-        self,
-        request: Optional[RequestSourceType] = None,
+            self,
+            request: Optional[RequestSourceType] = None,
     ):
         self.buffer = None
         self._pb_body = None
@@ -159,7 +159,7 @@ class DataRequest(Request):
 
     @property
     def proto_wo_data(
-        self,
+            self,
     ) -> Union['jina_pb2.DataRequestProtoWoData', 'jina_pb2.DataRequestProto']:
         """
         Transform the current buffer to a :class:`jina_pb2.DataRequestProtoWoData` unless the full proto has already
@@ -173,7 +173,7 @@ class DataRequest(Request):
 
     @property
     def proto(
-        self,
+            self,
     ) -> Union['jina_pb2.DataRequestProto', 'jina_pb2.DataRequestProtoWoData']:
         """
         Cast ``self`` to a :class:`jina_pb2.DataRequestProto` or a :class:`jina_pb2.DataRequestProto`. Laziness will be broken and serialization will be recomputed when calling.
@@ -187,7 +187,7 @@ class DataRequest(Request):
 
     @property
     def proto_with_data(
-        self,
+            self,
     ) -> 'jina_pb2.DataRequestProto':
         """
         Cast ``self`` to a :class:`jina_pb2.DataRequestProto`. Laziness will be broken and serialization will be recomputed when calling.
@@ -259,6 +259,7 @@ class DataRequest(Request):
     @property
     def parameters(self) -> Dict:
         """Return the `parameters` field of this DataRequest as a Python dict
+
         :return: a Python dict view of the parameters.
         """
         # if u get this u need to have it decompressed
@@ -267,6 +268,7 @@ class DataRequest(Request):
     @parameters.setter
     def parameters(self, value: Dict):
         """Set the `parameters` field of this Request to a Python dict
+
         :param value: a Python dict
         """
         self.proto_wo_data.parameters.Clear()
@@ -290,6 +292,35 @@ class DataRequest(Request):
         """
         return self.proto_wo_data.header.status
 
+    @property
+    def last_executor(self):
+        """
+        Returns the name of the last Executor that has processed this Request
+
+        :return: the name of the last Executor that processed this Request
+        """
+        if len(self.proto_wo_data.routes) > 0:
+            return self.proto_wo_data.routes[-1].executor
+
+    def add_executor(self, executor_name: str):
+        """
+        Adds Executor the the request routes
+
+        :param executor_name: name of the Executor processing the Request to be added to the routes
+        """
+        route_proto = jina_pb2.RouteProto()
+        route_proto.executor = executor_name
+        self.proto_wo_data.routes.append(route_proto)
+
+    @property
+    def routes(self):
+        """
+        Returns the routes from the request
+
+        :return: the routes object of this request
+        """
+        return self.proto_wo_data.routes
+
     @property
     def request_id(self):
         """
