@@ -1,5 +1,7 @@
 from typing import TYPE_CHECKING
 
+from jina.parsers.helper import _set_gateway_uses
+
 if TYPE_CHECKING:
     from argparse import Namespace
 
@@ -56,8 +58,8 @@ def executor_native(args: 'Namespace'):
 
     with runtime_cls(args) as rt:
         name = (
-            rt._data_request_handler._executor.metas.name
-            if hasattr(rt, '_data_request_handler')
+            rt._worker_request_handler._executor.metas.name
+            if hasattr(rt, '_worker_request_handler')
             else rt.name
         )
         rt.logger.info(f'Executor {name} started')
@@ -88,7 +90,7 @@ def worker_runtime(args: 'Namespace'):
 
     with WorkerRuntime(args) as runtime:
         runtime.logger.info(
-            f'Executor {runtime._data_request_handler._executor.metas.name} started'
+            f'Executor {runtime._worker_request_handler._executor.metas.name} started'
         )
         runtime.run_forever()
 
@@ -99,20 +101,14 @@ def gateway(args: 'Namespace'):
 
     :param args: arguments coming from the CLI.
     """
-    from jina.enums import GatewayProtocolType
     from jina.serve.runtimes import get_runtime
 
-    gateway_runtime_dict = {
-        GatewayProtocolType.GRPC: 'GRPCGatewayRuntime',
-        GatewayProtocolType.WEBSOCKET: 'WebSocketGatewayRuntime',
-        GatewayProtocolType.HTTP: 'HTTPGatewayRuntime',
-    }
-    runtime_cls = get_runtime(gateway_runtime_dict[args.protocol])
+    _set_gateway_uses(args)
+
+    runtime_cls = get_runtime('GatewayRuntime')
 
     with runtime_cls(args) as runtime:
-        runtime.logger.info(
-            f'Gateway with protocol {gateway_runtime_dict[args.protocol]} started'
-        )
+        runtime.logger.info(f'Gateway started')
         runtime.run_forever()
 
 
@@ -181,9 +177,9 @@ def hub(args: 'Namespace'):
     Start a hub builder for push, pull
     :param args: arguments coming from the CLI.
     """
-    from jina.hubble.hubio import HubIO
+    from hubble.executor.hubio import HubIO
 
-    getattr(HubIO(args), args.hub)()
+    getattr(HubIO(args), args.hub_cli)()
 
 
 def new(args: 'Namespace'):
@@ -210,3 +206,23 @@ def help(args: 'Namespace'):
     from jina_cli.lookup import lookup_and_print
 
     lookup_and_print(args.query.lower())
+
+
+def auth(args: 'Namespace'):
+    """
+    Authenticate a user
+    :param args: arguments coming from the CLI.
+    """
+    from hubble import api
+
+    getattr(api, args.auth_cli.replace('-', '_'))(args)
+
+
+def cloud(args: 'Namespace'):
+    """
+    Use jcloud (Jina Cloud) commands
+    :param args: arguments coming from the CLI.
+    """
+    from jcloud import api
+
+    getattr(api, args.jc_cli.replace('-', '_'))(args)
