@@ -6,11 +6,12 @@ ac_table = {
         'executor',
         'flow',
         'ping',
-        'dryrun',
         'export',
         'new',
         'gateway',
+        'auth',
         'hub',
+        'cloud',
         'help',
         'pod',
         'deployment',
@@ -24,27 +25,31 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
-            '--optout-telemetry',
             '--workspace-id',
             '--extra-search-paths',
             '--timeout-ctrl',
             '--k8s-namespace',
             '--polling',
+            '--shards',
+            '--replicas',
+            '--native',
             '--uses',
             '--uses-with',
             '--uses-metas',
             '--uses-requests',
             '--py-modules',
-            '--port-in',
-            '--host-in',
-            '--native',
             '--output-array-type',
+            '--exit-on-exceptions',
+            '--no-reduce',
+            '--disable-reduce',
+            '--grpc-server-options',
             '--entrypoint',
             '--docker-kwargs',
             '--volumes',
             '--gpus',
             '--disable-auto-volume',
             '--host',
+            '--host-in',
             '--quiet-remote-logs',
             '--upload-files',
             '--runtime-cls',
@@ -53,13 +58,18 @@ ac_table = {
             '--shard-id',
             '--pod-role',
             '--noblock-on-start',
-            '--shards',
-            '--replicas',
             '--port',
+            '--port-in',
             '--monitoring',
             '--port-monitoring',
             '--retries',
             '--floating',
+            '--tracing',
+            '--traces-exporter-host',
+            '--traces-exporter-port',
+            '--metrics',
+            '--metrics-exporter-host',
+            '--metrics-exporter-port',
             '--install-requirements',
             '--force-update',
             '--force',
@@ -67,7 +77,6 @@ ac_table = {
             '--uses-before-address',
             '--uses-after-address',
             '--connection-list',
-            '--disable-reduce',
             '--timeout-send',
         ],
         'flow': [
@@ -77,14 +86,20 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
-            '--optout-telemetry',
             '--workspace-id',
             '--uses',
             '--env',
             '--inspect',
         ],
-        'ping': ['--help', '--timeout', '--retries'],
-        'dryrun': ['--help', '--timeout'],
+        'ping': [
+            '--help',
+            'flow',
+            'executor',
+            'gateway',
+            '--timeout',
+            '--attempts',
+            '--min-successful-attempts',
+        ],
         'export flowchart': ['--help', '--vertical-layout'],
         'export kubernetes': ['--help', '--k8s-namespace'],
         'export docker-compose': ['--help', '--network_name'],
@@ -98,21 +113,12 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
-            '--optout-telemetry',
             '--workspace-id',
             '--extra-search-paths',
             '--timeout-ctrl',
             '--k8s-namespace',
-            '--polling',
-            '--uses',
-            '--uses-with',
-            '--uses-metas',
-            '--uses-requests',
-            '--py-modules',
-            '--port-in',
-            '--host-in',
-            '--native',
-            '--output-array-type',
+            '--entrypoint',
+            '--docker-kwargs',
             '--prefetch',
             '--title',
             '--description',
@@ -121,17 +127,23 @@ ac_table = {
             '--no-crud-endpoints',
             '--expose-endpoints',
             '--uvicorn-kwargs',
-            '--grpc-server-kwargs',
             '--ssl-certfile',
             '--ssl-keyfile',
             '--expose-graphql-endpoint',
             '--protocol',
+            '--protocols',
             '--host',
+            '--host-in',
             '--proxy',
-            '--port-expose',
+            '--uses',
+            '--uses-with',
+            '--py-modules',
+            '--grpc-server-options',
             '--graph-description',
             '--graph-conditions',
             '--deployments-addresses',
+            '--deployments-metadata',
+            '--deployments-no-reduce',
             '--deployments-disable-reduce',
             '--compression',
             '--timeout-send',
@@ -141,14 +153,28 @@ ac_table = {
             '--shard-id',
             '--pod-role',
             '--noblock-on-start',
-            '--shards',
-            '--replicas',
             '--port',
+            '--port-expose',
+            '--port-in',
+            '--ports',
             '--monitoring',
             '--port-monitoring',
             '--retries',
             '--floating',
+            '--tracing',
+            '--traces-exporter-host',
+            '--traces-exporter-port',
+            '--metrics',
+            '--metrics-exporter-host',
+            '--metrics-exporter-port',
         ],
+        'auth login': ['--help', '--force'],
+        'auth logout': ['--help'],
+        'auth token create': ['--help', '--expire'],
+        'auth token delete': ['--help'],
+        'auth token list': ['--help'],
+        'auth token': ['--help', 'create', 'delete', 'list'],
+        'auth': ['--help', 'login', 'logout', 'token'],
         'hub new': [
             '--help',
             '--name',
@@ -157,7 +183,7 @@ ac_table = {
             '--description',
             '--keywords',
             '--url',
-            '--add-dockerfile',
+            '--dockerfile',
         ],
         'hub push': [
             '--help',
@@ -181,7 +207,30 @@ ac_table = {
             '--force-update',
             '--force',
         ],
-        'hub': ['--help', 'new', 'push', 'pull'],
+        'hub status': ['--help', '--id', '--verbose', '--replay'],
+        'hub list': ['--help'],
+        'hub': ['--help', 'new', 'push', 'pull', 'status', 'list'],
+        'cloud login': ['--help'],
+        'cloud logout': ['--help'],
+        'cloud deploy': ['--help'],
+        'cloud list': ['--help', '--phase', '--name'],
+        'cloud status': ['--help', '--verbose'],
+        'cloud remove': ['--help'],
+        'cloud new': ['--help'],
+        'cloud survey': ['--help'],
+        'cloud': [
+            '--help',
+            '--version',
+            '--loglevel',
+            'login',
+            'logout',
+            'deploy',
+            'list',
+            'status',
+            'remove',
+            'new',
+            'survey',
+        ],
         'help': ['--help'],
         'pod': [
             '--help',
@@ -190,27 +239,31 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
-            '--optout-telemetry',
             '--workspace-id',
             '--extra-search-paths',
             '--timeout-ctrl',
             '--k8s-namespace',
             '--polling',
+            '--shards',
+            '--replicas',
+            '--native',
             '--uses',
             '--uses-with',
             '--uses-metas',
             '--uses-requests',
             '--py-modules',
-            '--port-in',
-            '--host-in',
-            '--native',
             '--output-array-type',
+            '--exit-on-exceptions',
+            '--no-reduce',
+            '--disable-reduce',
+            '--grpc-server-options',
             '--entrypoint',
             '--docker-kwargs',
             '--volumes',
             '--gpus',
             '--disable-auto-volume',
             '--host',
+            '--host-in',
             '--quiet-remote-logs',
             '--upload-files',
             '--runtime-cls',
@@ -219,13 +272,18 @@ ac_table = {
             '--shard-id',
             '--pod-role',
             '--noblock-on-start',
-            '--shards',
-            '--replicas',
             '--port',
+            '--port-in',
             '--monitoring',
             '--port-monitoring',
             '--retries',
             '--floating',
+            '--tracing',
+            '--traces-exporter-host',
+            '--traces-exporter-port',
+            '--metrics',
+            '--metrics-exporter-host',
+            '--metrics-exporter-port',
             '--install-requirements',
             '--force-update',
             '--force',
@@ -233,7 +291,6 @@ ac_table = {
             '--uses-before-address',
             '--uses-after-address',
             '--connection-list',
-            '--disable-reduce',
             '--timeout-send',
         ],
         'deployment': [
@@ -243,27 +300,31 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
-            '--optout-telemetry',
             '--workspace-id',
             '--extra-search-paths',
             '--timeout-ctrl',
             '--k8s-namespace',
             '--polling',
+            '--shards',
+            '--replicas',
+            '--native',
             '--uses',
             '--uses-with',
             '--uses-metas',
             '--uses-requests',
             '--py-modules',
-            '--port-in',
-            '--host-in',
-            '--native',
             '--output-array-type',
+            '--exit-on-exceptions',
+            '--no-reduce',
+            '--disable-reduce',
+            '--grpc-server-options',
             '--entrypoint',
             '--docker-kwargs',
             '--volumes',
             '--gpus',
             '--disable-auto-volume',
             '--host',
+            '--host-in',
             '--quiet-remote-logs',
             '--upload-files',
             '--runtime-cls',
@@ -272,12 +333,18 @@ ac_table = {
             '--shard-id',
             '--pod-role',
             '--noblock-on-start',
-            '--shards',
-            '--replicas',
             '--port',
+            '--port-in',
             '--monitoring',
+            '--port-monitoring',
             '--retries',
             '--floating',
+            '--tracing',
+            '--traces-exporter-host',
+            '--traces-exporter-port',
+            '--metrics',
+            '--metrics-exporter-host',
+            '--metrics-exporter-port',
             '--install-requirements',
             '--force-update',
             '--force',
@@ -285,24 +352,29 @@ ac_table = {
             '--uses-before-address',
             '--uses-after-address',
             '--connection-list',
-            '--disable-reduce',
             '--timeout-send',
             '--uses-before',
             '--uses-after',
             '--when',
             '--external',
+            '--grpc-metadata',
             '--deployment-role',
             '--tls',
-            '--port-monitoring',
         ],
         'client': [
             '--help',
             '--host',
+            '--host-in',
             '--proxy',
             '--port',
             '--tls',
             '--asyncio',
-            '--return-responses',
+            '--tracing',
+            '--traces-exporter-host',
+            '--traces-exporter-port',
+            '--metrics',
+            '--metrics-exporter-host',
+            '--metrics-exporter-port',
             '--protocol',
         ],
     },
